@@ -1,12 +1,9 @@
-use crate::InstrumentationLibrary;
-use opentelemetry_api::trace::OrderMap;
-use opentelemetry_api::{
+use opentelemetry::{
     trace::{
         Link, SamplingDecision, SamplingResult, SpanKind, TraceContextExt, TraceId, TraceState,
     },
-    Context, Key, Value,
+    Context, KeyValue,
 };
-use std::convert::TryInto;
 
 #[cfg(feature = "jaeger_remote_sampler")]
 mod jaeger_remote;
@@ -58,12 +55,12 @@ use opentelemetry_http::HttpClient;
 /// MUST NOT allow this combination.
 ///
 /// [OpenTelemetry SDK]: https://github.com/open-telemetry/opentelemetry-specification/blob/main/specification/trace/sdk.md#sampling
-/// [`SpanContext`]: opentelemetry_api::trace::SpanContext
-/// [`SpanContext::trace_flags()`]: opentelemetry_api::trace::SpanContext#method.trace_flags
+/// [`SpanContext`]: opentelemetry::trace::SpanContext
+/// [`SpanContext::trace_flags()`]: opentelemetry::trace::SpanContext#method.trace_flags
 /// [`SpanExporter`]: crate::export::trace::SpanExporter
 /// [`SpanProcessor`]: crate::trace::SpanProcessor
-/// [`Span`]: opentelemetry_api::trace::Span
-/// [`Span::is_recording()`]: opentelemetry_api::trace::Span#tymethod.is_recording
+/// [`Span`]: opentelemetry::trace::Span
+/// [`Span::is_recording()`]: opentelemetry::trace::Span#tymethod.is_recording
 pub trait ShouldSample: CloneShouldSample + Send + Sync + std::fmt::Debug {
     /// Returns the [`SamplingDecision`] for a [`Span`] to be created.
     ///
@@ -71,7 +68,7 @@ pub trait ShouldSample: CloneShouldSample + Send + Sync + std::fmt::Debug {
     /// make a decision about whether or not a [`Span`] should or should not be sampled. However,
     /// there are performance implications on the creation of a span
     ///
-    /// [`Span`]: opentelemetry_api::trace::Span
+    /// [`Span`]: opentelemetry::trace::Span
     /// [`should_sample`]: ShouldSample::should_sample
     #[allow(clippy::too_many_arguments)]
     fn should_sample(
@@ -80,9 +77,8 @@ pub trait ShouldSample: CloneShouldSample + Send + Sync + std::fmt::Debug {
         trace_id: TraceId,
         name: &str,
         span_kind: &SpanKind,
-        attributes: &OrderMap<Key, Value>,
+        attributes: &[KeyValue],
         links: &[Link],
-        instrumentation_library: &InstrumentationLibrary,
     ) -> SamplingResult;
 }
 
@@ -159,7 +155,7 @@ impl Sampler {
     where
         C: HttpClient + 'static,
         Sampler: ShouldSample,
-        R: crate::trace::TraceRuntime,
+        R: crate::runtime::RuntimeChannel,
         Svc: Into<String>,
     {
         JaegerRemoteSamplerBuilder::new(runtime, http_client, default_sampler, service_name)
@@ -173,9 +169,8 @@ impl ShouldSample for Sampler {
         trace_id: TraceId,
         name: &str,
         span_kind: &SpanKind,
-        attributes: &OrderMap<Key, Value>,
+        attributes: &[KeyValue],
         links: &[Link],
-        instrumentation_library: &InstrumentationLibrary,
     ) -> SamplingResult {
         let decision = match self {
             // Always sample the trace
@@ -195,7 +190,6 @@ impl ShouldSample for Sampler {
                                 span_kind,
                                 attributes,
                                 links,
-                                instrumentation_library,
                             )
                             .decision
                     },
@@ -214,15 +208,7 @@ impl ShouldSample for Sampler {
             #[cfg(feature = "jaeger_remote_sampler")]
             Sampler::JaegerRemote(remote_sampler) => {
                 remote_sampler
-                    .should_sample(
-                        parent_context,
-                        trace_id,
-                        name,
-                        span_kind,
-                        attributes,
-                        links,
-                        instrumentation_library,
-                    )
+                    .should_sample(parent_context, trace_id, name, span_kind, attributes, links)
                     .decision
             }
         };
@@ -263,8 +249,7 @@ pub(crate) fn sample_based_on_probability(prob: &f64, trace_id: TraceId) -> Samp
 mod tests {
     use super::*;
     use crate::testing::trace::TestSpan;
-    use crate::trace::{Sampler, ShouldSample};
-    use opentelemetry_api::trace::{SamplingDecision, SpanContext, SpanId, TraceFlags, TraceState};
+    use opentelemetry::trace::{SpanContext, SpanId, TraceFlags};
     use rand::Rng;
 
     #[rustfmt::skip]
@@ -339,16 +324,15 @@ mod tests {
                     None
                 };
 
-                let trace_id = TraceId::from(rng.gen::<[u8; 16]>());
+                let trace_id = TraceId::from(rng.gen::<u128>());
                 if sampler
                     .should_sample(
                         parent_context.as_ref(),
                         trace_id,
                         name,
                         &SpanKind::Internal,
-                        &Default::default(),
                         &[],
-                        &InstrumentationLibrary::default(),
+                        &[],
                     )
                     .decision
                     == SamplingDecision::RecordAndSample
@@ -381,19 +365,18 @@ mod tests {
     #[test]
     fn clone_a_parent_sampler() {
         let sampler = Sampler::ParentBased(Box::new(Sampler::AlwaysOn));
+        #[allow(clippy::redundant_clone)]
         let cloned_sampler = sampler.clone();
 
         let cx = Context::current_with_value("some_value");
-        let instrumentation_library = InstrumentationLibrary::default();
 
         let result = sampler.should_sample(
             Some(&cx),
             TraceId::from_u128(1),
             "should sample",
             &SpanKind::Internal,
-            &Default::default(),
             &[],
-            &instrumentation_library,
+            &[],
         );
 
         let cloned_result = cloned_sampler.should_sample(
@@ -401,9 +384,8 @@ mod tests {
             TraceId::from_u128(1),
             "should sample",
             &SpanKind::Internal,
-            &Default::default(),
             &[],
-            &instrumentation_library,
+            &[],
         );
 
         assert_eq!(result, cloned_result);
@@ -447,15 +429,13 @@ mod tests {
 
         for (name, delegate, parent_cx, expected) in test_cases {
             let sampler = Sampler::ParentBased(Box::new(delegate));
-            let instrumentation_library = InstrumentationLibrary::default();
             let result = sampler.should_sample(
                 Some(&parent_cx),
                 TraceId::from_u128(1),
                 name,
                 &SpanKind::Internal,
-                &Default::default(),
                 &[],
-                &instrumentation_library,
+                &[],
             );
 
             assert_eq!(result.decision, expected);
